package com.dummies.silentmodetoggle.widget;

import android.app.Activity;
import android.app.IntentService;
import android.app.PendingIntent;
import android.appwidget.AppWidgetManager;
import android.content.ComponentName;
import android.content.Intent;
import android.media.AudioManager;
import android.widget.RemoteViews;

import com.dummies.silentmodetoggle.R;
import com.dummies.silentmodetoggle.util.RingerHelper;

/**
 * Der Service, der alle Operationen unseres Widgets verarbeitet.
 * Der an den Service geschickte Intent teilt ihm mit, was zu tun ist.
 *
 * Dies ist eine Instanz von IntentService.  Ein IntentService ist eine
 * praktische Methode, Dinge zu verarbeiten, die in Hintergrund-Threads
 * erledigt werden mssen. Wenn ein neuer Intent empfangen wird, wird
 * onHandleIntent in einem Hintergrund-Thread ausgefhrt. Auf diese
 * Weise knnen Sie beliebige Operationen im Hintergrund ausfhren,
 * unabhngig davon, wie lange diese womglich dauern, ohne den
 * UI-Thread im Vordergrund zu blockieren (wodurch sich die
 * App aufngen wrde).
 */
public class AppWidgetService extends IntentService {

    // Ein Flag, das in unserem Intent gesetzt wird, wenn wir
    // mitteilen wollen, dass wir die Stummschaltung des Telefons
    // umschalten wollen
    private static String ACTION_DO_TOGGLE = "actionDoToggle";

    AudioManager audioManager;

    /**
     * Alle IntentServices brauchen einen Namen. Unserer heit
     * "AppWidgetService"
     */
    public AppWidgetService() {
        super("AppWidgetService");
    }

    /**
     * Dies wird aufgerufen, wenn der Service initialisiert wird,
     * nach dem Java-Konstruktor des Objekts.
     */
    @Override
    public void onCreate() {
        // Immer super.onCreate aufrufen
        super.onCreate();

        // Genau wie in unserer Aktivitt erhalten wir eine Referenz auf den
        // AudioManager von Android, wir knnen ihn also verwenden, um unsere
        // Klingel umzuschalten.
        audioManager = (AudioManager) getSystemService(
                Activity.AUDIO_SERVICE);
    }

    /**
     * onHandleIntent wird in einem Hintergrund-Thread aufgerufen.
     * @param intent Der zu verarbeitende Intent
     */
    @Override
    protected void onHandleIntent(Intent intent) {

        // Den Intent prfen. Wenn er ACTION_DO_TOGGLE enthlt,
        // den Stummschaltungsmodus des Telefons umschalten.
        // Wenn er nicht ACTION_DO_TOGGLE enthlt, ist dies nur
        // eine Update-Anforderung, deshalb die UI aktualisieren.
        if( intent!=null && intent.getBooleanExtra(
                ACTION_DO_TOGGLE,false)) {
            RingerHelper.performToggle(audioManager);
        }

        // Eine Referenz auf den AppWidgetManager von Android erhalten
        AppWidgetManager mgr = AppWidgetManager.getInstance(this);

        // Die UI des Widgets aktualisieren
        // Zuerst den Namen fr unser Widget finden,
        // dann den AppWidgetManager auffordern, ihn zu aktualisieren,
        // die Ansichten verwenden, die wir in update() erstellen
        ComponentName name = new ComponentName(this, AppWidget.class);
        mgr.updateAppWidget(name, updateUi());
    }

    /**
     * Gibt die RemoteViews zurck, mit denen das Widget aktualisiert wird.
     * hnlich wie updateUi in MainActivity, aber fr die Verwendung mit
     * Widgets geeignet.
     */
    private RemoteViews updateUi() {
        // Die Layoutdatei res/layout/app_widget.xml erweitern zu einem
        // RemoteViews-Objekt, das verwendet wird, um mit unserem Widget
        // zu kommunizieren.
        RemoteViews remoteViews = new RemoteViews(getPackageName(),
                R.layout.app_widget);

        // Festlegen, welches Bild in unserem Widget verwendet wird
        int phoneImage = RingerHelper.isPhoneSilent(audioManager)
                ? R.drawable.icon_ringer_off
                : R.drawable.icon_ringer_on;

        // Das entsprechende Bild festlegen
        remoteViews.setImageViewResource(R.id.phone_state, phoneImage);

        // Einen Intent erstellen, um den Status des Telefons umzuschalten.
        // Dieser Intent gibt ACTION_DO_TOGGLE=true an,
        // wonach wir in onHandleIntent suchen.
        Intent intent = new Intent(this, AppWidgetService.class)
                .putExtra(ACTION_DO_TOGGLE,true);

        // Den Intent in einen PendingIntent umwandeln. Ein PendingIntent
        // gibt jemandem in einem anderen Prozess die Erlaubnis, uns
        // einen Intent zu senden.
        // In diesem Fall luft das Widget tatschlich in einem anderen
        // Prozess (dem Launcher-Prozess des Telefons),
        // es braucht also einen Pending Intent, um ebenfalls mit
        // unserem Service kommunizieren zu knnen.
        // Wir geben FLAG_ONE_SHOT fr diesen Intent an, um sicherzustellen,
        // dass er nur einmal verwendet wird. Es gibt Situationen, wo ein
        // PendingIntent automatisch fr uns erneut aufgerufen wird,
        // und wir wollen sicherstellen,d ass wir nicht versehentlich ein
        // paar berflssige Umschaltungen vornehmen. Weitere Informationen
        // ber Pending Intents unter
        // http://d.android.com/reference/android/app/PendingIntent.html
        PendingIntent pendingIntent =
                PendingIntent.getService(this, 0, intent,
                        PendingIntent.FLAG_ONE_SHOT);

        // Das Layout fr das App Widget erhalten und einen On-Click-
        // Listener fr die Schaltflche aktivieren
        remoteViews.setOnClickPendingIntent(R.id.phone_state, pendingIntent);

        return remoteViews;
    }
}
