package com.dummies.tasks;

import android.app.Activity;
import android.os.Bundle;
import android.support.wearable.view.WearableListView;
import android.util.Log;

import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.api.GoogleApiClient;
import com.google.android.gms.common.api.ResultCallback;
import com.google.android.gms.common.data.FreezableUtils;
import com.google.android.gms.wearable.DataApi;
import com.google.android.gms.wearable.DataEventBuffer;
import com.google.android.gms.wearable.DataItem;
import com.google.android.gms.wearable.DataItemBuffer;
import com.google.android.gms.wearable.Wearable;

import java.util.List;

/**
 * Die Hauptaktivitt fr diese app.
 * Diese Aktivitt implementiert die DataApi.DataListener-Schnittstelle,
 * um ber Sync-Ereignisse informiert zu werden. Auerdem implementiert
 * sie die GoogleApiClient ConnectionCallbacks und OnConnectionFailedListener,
 * um Connectivity-Ereignisse zu verarbeiten.
 *
 * Wenn Sie eine App schreiben, die synchron im Hintergrund laufen muss
 * (unsere macht das nicht), sollten Sie einen WearableListenerService
 * verwenden, der stetig im Hintergrund luft.
 */
public class MainActivity extends Activity
    implements DataApi.DataListener, GoogleApiClient.ConnectionCallbacks, GoogleApiClient.OnConnectionFailedListener
{

    /**
     * Wearables verwenden den GoogleApiClient fr die Kommunikation mit
     * Host-Gerten (das ist in der Regel Ihr Telefon).
     */
    GoogleApiClient googleApiClient;

    /**
     * Ein WearableListAdapter, der die Task-Liste lesen kann.
     */
    WearableTaskListAdapter adapter;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        // Den GoogleApiClient einrichten.
        // Ihm mitteilen, dass wir das Wearable.API bentigen, und auch,
        // dass es dazu ein Callback auf "this" fr alle Connectivity-
        // Benachrichtigungen ausfhren soll.
        googleApiClient = new GoogleApiClient.Builder(this)
            .addApi(Wearable.API)
            .addConnectionCallbacks(this)
            .addOnConnectionFailedListener(this)
            .build();

        // Den Adapter einrichten
        adapter = new WearableTaskListAdapter(this);

        // Die WearableListView in unserer View-Hierarchie finden
        WearableListView listView =
            (WearableListView) findViewById(R.id.list);

        // Adapter und listView verknpfen
        listView.setAdapter(adapter);
    }

    @Override
    protected void onStart() {
        super.onStart();

        // Verbindung mit dem GoogleApiClient.  Wenn wir verbunden sind,
        // erhalten wir einen Callback auf onConnected().
        googleApiClient.connect();
    }

    @Override
    protected void onStop() {
        super.onStop();

        // Wir sind fertig, deshalb vom GoogleApiClient trennen.
        googleApiClient.disconnect();
    }

    @Override
    public void onConnected(Bundle bundle) {
        Log.d("MainActivity", "onConnected");

        // Den Adapter aktualisieren, sobald wir verbunden sind.
        updateList();

        // Anmeldung fr weitere Daten-Updates. Es erfolgt ein Callback
        // auf onDataChanged, wenn etwas aktualisiert wird.
        Wearable.DataApi.addListener(googleApiClient, this);
    }

    @Override
    public void onDataChanged(DataEventBuffer dataEvents) {
        Log.d("MainActivity", "onDataChanged");

        // Immer die dataEvents freigeben, wenn wir fertig sind.
        // In diesem Fall verwenden wir die dataEvents direkt, deshalb
        // geben wir sie direkt frei.
        dataEvents.release();

        // Es wurde uns mitgeteilt, dass es ein Update gab, deshalb
        // aktualisieren wir unseren Adapter.
        updateList();
    }

    @Override
    public void onConnectionSuspended(int i) {
        // Einfach eine Nachricht protokollieren. Wir mssen berhaupt nichts tun,
        // auer eine Nachricht zu protokollieren, um Fehler leichter zu erkennen.
        Log.d("MainActivity", "onConnectionSuspended");
    }

    @Override
    public void onConnectionFailed(ConnectionResult connectionResult) {
        // Einfach eine Nachricht protokollieren. Wir mssen berhaupt nichts tun,
        // auer eine Nachricht zu protokollieren, um Fehler leichter zu erkennen.
        Log.d("MainActivity", "onConnectionFailed");
    }

    private void updateList() {
        // Die komplette Liste der DataItems mit DataApi.getDataItems laden.
        // Dafr ist mglicherweise eine Netzwerk-Synchronisierung erforderlich,
        // was einige Zeit dauert, deshalb die Ergebnisse in einem ResultCallback
        // spter erhalten.
        Wearable.DataApi.getDataItems(googleApiClient).setResultCallback(
            new ResultCallback<DataItemBuffer>() {
                @Override
                public void onResult(DataItemBuffer dataItems) {
                    try {
                        // Bevor DataItems verwendet werden, mssen sie
                        // "eingefroren" werden, damit sie sich nicht ndern,
                        // bevor Sie sie durchlaufen.
                        List<DataItem> items
                            = FreezableUtils.freezeIterable(dataItems);

                        // Den Adapter mit den neuen Elementen aktualisieren.
                        adapter.setResults(items);

                        Log.d("MainActivity", "adapter.setResults");
                    } finally {

                        // Immer die DataItems freigeben, wenn Sie fertig sind.
                        dataItems.release();
                    }
                }
            });
    }
}
